#
# Copyright (c) Daniel Sheffield 2024
# All rights reserved
#
# THIS SOFTWARE IS PROVIDED AS IS WITHOUT WARRANTY
from io import BytesIO
import time
from bottle import (
    Bottle,
    default_app,
    route, request, response,
    redirect, abort,
    template, static_file,
    HTTPResponse,
)
from itertools import chain
from base64 import urlsafe_b64decode as b64decode
from linkpreview import link_preview

from .validate import CLIP_SIZE_LIMIT, get_file_mimetype, get_file_size, get_filename, validate_file
from .hash_util import B32REGEX, normalize_base32, blake, bytes_to_base32
from .save import save_upload
from .qr import get_qr_code
from json import dumps, load

from sqlite3 import connect, Row

SCHEME = "https://"
HOST = ""
DOMAIN = "shandan.one"
PORT = ""
LOCATION = SCHEME + (f"{HOST}." if HOST else "") + DOMAIN + (f":{PORT}" if PORT else "")

@route('/preview', method=['GET'])
def get_preview():
    link = request.params.link
    if not link:
        return dumps(None)
    try:
        page = link_preview(link, parser="lxml")
        return dumps({
            'title': page.title,
            'img': page.absolute_image,
            'domain': page.site_name,
            'link': link,
        })
    except:
        return dumps(None)

@route('/hash', method=['POST'])
def get_hash():
    data = dict(map(
        lambda x: (x[0], x[1].encode('utf-8')),
        load(request.body).items()
    ))
    _bytes = blake(**data)
    return bytes_to_base32(_bytes)

@route('/normalize', method=['GET'])
def normalize():
    _hash = request.params.hash
    response.content_type = 'application/json'
    return dumps({
        'i': _hash,
        'o': normalize_base32(_hash) if _hash else None,
    })

@route('/qr', method=['POST'])
def get_qr():
    data = load(request.body)
    return get_qr_code(**data).decode('utf-8')

@route('/static/<filename:path>')
def send_static(filename):
    return static_file(filename, root='rest/static')

@route('/<route:re:(clip|goto|upload)>/open')
def _get_clip(route):
    return redirect(f'/{route}/open.sql')

@route('/<route:re:(clip|goto|upload)>', method=['GET', 'POST'])
def clip(route):
    return redirect(f'/{route}.sql')

@route('/<route:re:(clip|goto)>/<hash:re:{B32REGEX}{{1,5}}>', method='GET')
def get_clip(route, filename):
    return redirect(f'/{route}.sql?hash={filename}&go=true')

@route(f'/upload/<hash:re:{B32REGEX}{{1,5}}>', method='GET')
def get_upload(hash, db=None):
    hash = hash and normalize_base32(hash)
    con = connect('util.db')
    fname, mimetype, content = (None, None, None)
    try:
        fname, mimetype, content = con.cursor().execute(f"""
SELECT name, mime, content
FROM upload
WHERE hash = '{hash}'
LIMIT 1;
""").fetchall()[0]
    finally:
        con.close()

    # extract bytes from data url
    _, mime, encoding, content = chain.from_iterable(map(
        lambda x: x.split(','), chain.from_iterable(map(
            lambda x: x.split(';'), content.split(':')
        ))
    ))
    assert encoding == 'base64', f'unsupported encoding: {encoding}'
    content = b64decode(content + '==')

    if request.params.download == "false":
        download = False
    headers = {}
    headers['Content-Length'] = len(content)

    # TODO: create ext from mime type?
    headers['Content-Disposition'] = 'attachment; filename="%s"' % (fname or hash)
    headers['Content-Encoding'] = 'application/octet-stream'
    # if mimetype == 'auto':
    #     mimetype, encoding = mimetypes.guess_type(filename)
    #     if encoding: headers['Content-Encoding'] = encoding

    # if mimetype:
    #     if mimetype[:5] == 'text/' and charset and 'charset' not in mimetype:
    #         mimetype += '; charset=%s' % charset
    #     headers['Content-Type'] = mimetype
    #lm = time.strftime("%a, %d %b %Y %H:%M:%S GMT", time.gmtime(created))
    #headers['Last-Modified'] = lm
    return HTTPResponse(content, **headers)

@route('/<any>/', method='GET')
def redirect_trailing_slash(any): return redirect(f'/{any}')
